
/*********************************************************************
This file is part of the supergetopt library. It contains the definitions
of the supergetopt library functions as well as some useful string
manipulation functions.

Copyright (C) 1997, 1998, 1999 A.P. Russo (apr@planet.net)

The supergetopt library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
USA.
*********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "superget.h"

static char *shInterpreter(char *s);

int skipspace(char *x)    /* skip over space to next word. If at word, skip it too  */

{
    register int j, first;
    int lenx;
	
	lenx = strlen(x);
	if( lenx < 0 )
	{
		fprintf(stderr, "Bad string <%s>\n",x);
		return(-1000);
	}
 
    for ( j = 0 , first = 0 ; j < lenx ; j++ )
    {
        if ( ( *(x+j) ) == ' ' )
        {
            if( first == 0 )
                first = 1;
        }
        else
        {   
            if( first == 1 )
                return(j);
        }
    }    
 
    return(-1);
}
 
int strgrep(char *x, char *y)    /* grep x for y */
{
    register int j;
    register int lenx = strlen(x);
    register int leny = strlen(y);
 
    for(j=0; j<lenx; j++)
        if( strncmp( x+j,y,leny ) == 0 ) return(j);
 
    return(-1);
}

int make_args(char *line, char **argv) 
		/* medium -- delimiters are spaces, tabs, =, comma, semicolon */
		/* "C" style comments are removed */
		/* # shell style comments are removed */
		/* Environment variables are expanded if they exist */

{
	return( make_args_custom(line,argv,"\t = , ;") );
}

int args_to_string(char *string, int size, int argc, char **argv)
{
	register int i, tot;

	for( i = 1, tot=strlen(argv[0]), strcpy(string,argv[0]); i<argc && tot < size-1; i++)
	{
		strcat(string," ");
		strcat(string,argv[i]);
		tot += strlen(argv[i])+1;
	}

	return(1);
}

int make_args_custom(char *line, char **argv, char *delims) /* also removes C-style comments 
									like this one and shell style comments
									like this one # hi 
									Also expands environ variables
									according to unix shell conventions */
{
	int argcount, k;
	int comment;
	static char *envar;
	argcount = -1;

	if( (k=strgrep(line, "#" )) >= 0 )
		line[k] = '\0';	/* kill at shell style "#" comment */

	if( ( k = strgrep(line,"\n") ) != -1 )
		line[k] = '\0';         /* remove the return so not considered an arg */

	if( ( k = strgrep(line,"\r") ) != -1 )
		line[k] = '\0';         /* remove the dos return so not considered an arg */


/* 	fprintf(stderr,"input line <%s>. last char was: 0x%x\n",line,line[strlen(line)-1]); 
 */
    for(comment = 0, argcount= -1, argv[++argcount] = strtok(line,delims) ; ; )
	{
		if( argv[argcount] == NULL ) break;
		if( (argcount >= 0 && strcmp("/*",argv[argcount]) == 0) || comment == 1 )
		{
			comment = 1;
			argcount--;	/* overwrite arg until hit comment end */
		}

		if( (argv[++argcount]=strtok( (char *) NULL,delims)) == NULL )
			break;

		if( comment != 1 )	/* expand env variables */
		{
			if( strchr(argv[argcount],(int) '$') != NULL ) 	/* expand if env variable */
			{
				envar = shInterpreter( argv[argcount] );
				argv[argcount][strlen(argv[argcount])] = ' ';
				strtok(argv[argcount],delims);
				if( envar != NULL )
					argv[argcount] = (char *) strdup(envar);
			}
		}
				
		if( strcmp("*/",argv[argcount]) == 0 ) 	/* end! */
		{
			argcount--;
			comment = 0;
		}
	}

/* 	print_argv(argcount,argv); */
	return(argcount);
}

void breakline(char *line, int columns)
{
    register int i, linelen;

    while ( (linelen = strlen(line)) > columns )
    {
        i = columns+1;
        while( line[--i] != ' ' );
        line[i] = '\n';     /* put in new line */
        line += i+1;
    }
}

static char *shInterpreter(char *s)
{
	static char delims[] = { "/-" }; /* used to isolate env variables */
	char *token, *envar;
	static char scopy[500], expanded[500];
	int cindex;
	char *missing;

/* 	fprintf(stderr,"s=<%s>\n",s); */

	memset( expanded, 0, 500 );
	memset( scopy, 0, 500 );

	strcpy(scopy,s);

	for( token = strtok(scopy,delims), expanded[0] = '\0', cindex = 0, missing = s ; token != NULL ; )
	{
		cindex += (int) token - (int) scopy + strlen(token);
		missing += (strlen(token));

/* 		fprintf(stderr,"cindex=%d token=<%s> missing=<%c>\n",cindex,token,*missing); */
		if( token[0] == '$' ) 	/* env variable */
		{
			envar = getenv( &token[1] );
			if( envar != NULL ) 
				strcat(expanded,envar);
			expanded[strlen(expanded)] = s[cindex];
/* 			fprintf(stderr,"added <%c> %d to exp <%s> scp=0x%x tok=0x%x\n",*missing,(int)s-(int)missing,expanded,scopy,token); */
			missing++;
		}
		else
		{
			strcat(expanded,token);
			expanded[strlen(expanded)] = *missing;
/* 			fprintf(stderr,"Added <%c> %d to exp <%s> scp=0x%x tok=0x%x\n",*missing,(int)s-(int)missing,expanded,scopy,token); */
			missing++;
		}

		if( (token = strtok( (char *) NULL,delims)) == NULL )
			break;
		else
		{
			if( token[0] == '$' ) 	/* env variable */
			{
				expanded[strlen(expanded)] = *missing;
/* 				fprintf(stderr,"_dded <%c> %d to exp. scp=0x%x tok=0x%x\n",*missing,(int)s-(int)missing,scopy,token); */
			}
		}
	}

/* 	fprintf(stderr,"e=<%s>\n",expanded); */

	return( (char *) strdup(expanded) ); /* so subsequent calls don't change anything */
}


