/*********************************************************************
This file is part of the supergetopt library. It contains the definitions
of the supergetopt library functions as well as some useful string
manipulation functions.

Copyright (C) 1997, 1998, 1999 A.P. Russo (apr@planet.net)

The supergetopt library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
USA.
*********************************************************************/

#include <stdio.h>
#include "superget.h"

/* local static functions */
static void bobsled(char *c, double *d1, char *s, int *i);
static void toast(short int *x, short int *y);
static void bingo(float *floats, int numfloats);
static void stringo(char **strings, int numstrings);
static void dingo(double *f);
static void charro(char *chars, int numchars);
static void wildcardfunc(int *x, int *y);
static void usage();

int main( int argc, char *argv[] )
{
	int n;

/* example call to supergetopt */

	n = supergetopt(argc,argv,
			"-bobhope: %c %lf %s %d",bobsled, /* mixed types: char double string and int */
			"-toast:%hd %hd", toast, /* shorts */
			"-bingo:*%f", bingo, /* arb number of floats */
			"-stringo:*%s",stringo, /* as many strings as you like until next flag */
			"-dingo:%lf",dingo, /* just one double */
			"-charro:*%c",charro, /* arb number of characters */
			"-help", usage,	/* no args to this flag  */
			":%d%d", wildcardfunc, (char * ) 0 ); /* flagless args */
	
	printf("Supergetopt returned %d\n", n);
	
	return(0);
}

static void usage()
{
printf("testSuperGet: [options] [<integer> <integer>]\n  Options:\n\
\t[-bobhope: <char> <double> <string> <int>]\n\
\t[-toast: <short> <short>]\n\
\t[-bingo: <float1> ... <floatN>]\n\
\t[-stringo: <string1> ... <stringN>]\n\
\t[-dingo: <double>]\n\
\t[-charro: <char>]\n\
\t[-help]\n"
 );
}

static void toast(short int *x, short int *y)
{
	printf("Toast: I have  %d pieces with %d slabs\n",*x,*y);
}

static void bobsled(char *c, double *d1, char *s, int *i)
{
	printf("bobsled: I got %c and %f and %d and <%s>\n",*c, *d1,*i, s);
}

static void bingo(float *floats, int numfloats)
{
	register int i;
	printf("Bingo: numfloats=%d\n",numfloats);

	for( i = 0 ; i < numfloats ; i++ )
		printf("\tbingo: got %f\n",floats[i]);

}
static void dingo(double *f)
{
	register int i;
	
	printf("dingo: got %f as a double\n", *f);

}
static void charro(char *chars, int numchars)
{
	register int i;
	printf("Charro: numchars=%d\n",numchars);

	for( i = 0 ; i < numchars ; i++ )
		printf("\tcharro: got <%c>\n",chars[i]);

}

static void stringo(char **strings, int numstrings)
{
	register int i;
	printf("Stringo: numstrings=%d\n",numstrings);

	for( i = 0 ; i < numstrings ; i++ )
		printf("\tstringo: got <%s>\n",strings[i]);

}

static void wildcardfunc(int *x, int *y)
{
	printf("Wildcardfunc: got integers %d and %d\n",*x,*y);
}


